=== modified file 'bzrlib/osutils.py'
--- bzrlib/osutils.py	2006-07-13 19:58:22 +0000
+++ bzrlib/osutils.py	2006-07-31 04:37:47 +0000
@@ -875,13 +875,31 @@
     to exclude some directories, they are then not descended into.
     
     The data yielded is of the form:
-    [(relpath, basename, kind, lstat, path_from_top), ...]
+    ((directory-relpath, directory-path-from-top),
+    [(relpath, basename, kind, lstat), ...]),
+     - directory-relpath is the relative path of the directory being returned
+       with respect to top. prefix is prepended to this.
+     - directory-path-from-root is the path including top for this directory. 
+       It is suitable for use with os functions.
+     - relpath is the relative path within the subtree being walked.
+     - basename is the basename of the path
+     - kind is the kind of the file now. If unknown then the file is not
+       present within the tree - but it may be recorded as versioned. See
+       versioned_kind.
+     - lstat is the stat data *if* the file was statted.
+     - planned, not implemented: 
+       path_from_tree_root is the path from the root of the tree.
 
     :param prefix: Prefix the relpaths that are yielded with 'prefix'. This 
         allows one to walk a subtree but get paths that are relative to a tree
         rooted higher up.
     :return: an iterator over the dirs.
     """
+    #TODO there is a bit of a smell where the results of the directory-
+    # summary in this, and the path from the root, may not agree 
+    # depending on top and prefix - i.e. ./foo and foo as a pair leads to
+    # potentially confusing output. We should make this more robust - but
+    # not at a speed cost. RBC 20060731
     lstat = os.lstat
     pending = []
     _directory = _directory_kind
@@ -899,8 +917,10 @@
         for name in sorted(_listdir(top)):
             abspath = top + '/' + name
             statvalue = lstat(abspath)
-            dirblock.append ((relroot + name, name, file_kind_from_stat_mode(statvalue.st_mode), statvalue, abspath))
-        yield dirblock
+            dirblock.append((relroot + name, name,
+                file_kind_from_stat_mode(statvalue.st_mode),
+                statvalue, abspath))
+        yield (currentdir[0], top), dirblock
         # push the user specified dirs from dirblock
         for dir in reversed(dirblock):
             if dir[2] == _directory:

=== modified file 'bzrlib/tests/test_osutils.py'
--- bzrlib/tests/test_osutils.py	2006-07-13 19:58:22 +0000
+++ bzrlib/tests/test_osutils.py	2006-07-30 14:02:21 +0000
@@ -307,36 +307,40 @@
             ]
         self.build_tree(tree)
         expected_dirblocks = [
-                [
-                    ('0file', '0file', 'file'),
-                    ('1dir', '1dir', 'directory'),
-                    ('2file', '2file', 'file'),
-                ],
-                [
-                    ('1dir/0file', '0file', 'file'),
-                    ('1dir/1dir', '1dir', 'directory'),
-                ],
-                [
-                ],
+                (('', '.'),
+                 [('0file', '0file', 'file'),
+                  ('1dir', '1dir', 'directory'),
+                  ('2file', '2file', 'file'),
+                 ]
+                ),
+                (('1dir', './1dir'),
+                 [('1dir/0file', '0file', 'file'),
+                  ('1dir/1dir', '1dir', 'directory'),
+                 ]
+                ),
+                (('1dir/1dir', './1dir/1dir'),
+                 [
+                 ]
+                ),
             ]
         result = []
         found_bzrdir = False
-        for dirblock in osutils.walkdirs('.'):
+        for dirdetail, dirblock in osutils.walkdirs('.'):
             if len(dirblock) and dirblock[0][1] == '.bzr':
                 # this tests the filtering of selected paths
                 found_bzrdir = True
                 del dirblock[0]
-            result.append(dirblock)
+            result.append((dirdetail, dirblock))
 
         self.assertTrue(found_bzrdir)
         self.assertEqual(expected_dirblocks,
-            [[line[0:3] for line in block] for block in result])
+            [(dirinfo, [line[0:3] for line in block]) for dirinfo, block in result])
         # you can search a subdir only, with a supplied prefix.
         result = []
-        for dirblock in osutils.walkdirs('1dir', '1dir'):
+        for dirblock in osutils.walkdirs('./1dir', '1dir'):
             result.append(dirblock)
         self.assertEqual(expected_dirblocks[1:],
-            [[line[0:3] for line in block] for block in result])
+            [(dirinfo, [line[0:3] for line in block]) for dirinfo, block in result])
 
     def assertPathCompare(self, path_less, path_greater):
         """check that path_less and path_greater compare correctly."""

